// FunctionDefinition.cpp: implementation of the CFunctionDefinition class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FunctionDefinition.h"
#include "Commands.h"
#include "Utils.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFunctionDefinition::CFunctionDefinition(CString szSymbol, CString szDefinition)
{
	SetSymbol(szSymbol);
	SetDefinition(szDefinition);
}

void CFunctionDefinition::SetSymbol(CString szSymbol)
{
	m_szSymbol = szSymbol;
}

void CFunctionDefinition::SetDefinition(CString szDefinition)
{
	m_szDefinition = szDefinition;
}

CString CFunctionDefinition::GetSymbol()
{
	return m_szSymbol;
}

CString CFunctionDefinition::GetDefinition()
{
	return m_szDefinition;
}

void CFunctionDefinition::ClearKey(int nApplicationID)
{
	ASSERT(0 <= nApplicationID && nApplicationID < MAX_APP);

#ifdef MFCDLL
	while (!m_oKey[nApplicationID].IsEmpty()) {
		KeyBind *pKeyBind = (KeyBind *)m_oKey[nApplicationID].GetHead();
		delete pKeyBind;
//		pKeyBind = NULL;
		m_oKey[nApplicationID].RemoveHead();
	}
#else
	m_oKey[nApplicationID].clear();		///< @todo O`FbN
#endif
}

void CFunctionDefinition::SetKey(int nApplicationID, int nCommandType, int nKey)
{
	ASSERT(0 <= nApplicationID && nApplicationID < MAX_APP);

#ifdef MFCDLL
	try {
		KeyBind *pKeyBind = new KeyBind;

//		if (pKeyBind) {
			pKeyBind->nCommandType = nCommandType;
			pKeyBind->bVk = (BYTE)nKey;
			pKeyBind->nControlID = 0;

			m_oKey[nApplicationID].AddTail((CObject *)pKeyBind);
//		}
	}
	catch (CMemoryException* e) {
		e->Delete();
//		CUtils::Log("CFunctionDefinition::SetKey: 'new' threw an exception");
	}
#else
	KeyBind k = { nCommandType, (BYTE)nKey, 0 };
	m_oKey[nApplicationID].push_back(k);		///< @todo O`FbN
#endif
}

int CFunctionDefinition::GetKeyNumber(int nApplicationID)
{
	ASSERT(0 <= nApplicationID && nApplicationID < MAX_APP);

	return (int)m_oKey[nApplicationID].GetCount();
}

void CFunctionDefinition::GetKey(int nApplicationID, int nKeyID, int* pCommandType, int*pKey)
{
	ASSERT(0 <= nApplicationID && nApplicationID < MAX_APP);

	if (nKeyID < 0 || m_oKey[nApplicationID].GetCount() <= nKeyID) {
		*pCommandType = 0;
		*pKey = 0;
		return;
	}

#ifdef MFCDLL
	KeyBind *pKeyBind = (KeyBind*)m_oKey[nApplicationID].GetAt(m_oKey[nApplicationID].FindIndex(nKeyID));
	*pCommandType = pKeyBind->nCommandType;
	*pKey = pKeyBind->bVk;
#else
	const KeyBind& k = m_oKey[nApplicationID][nKeyID];
	*pCommandType = k.nCommandType;
	*pKey = k.bVk;
#endif
}

void CFunctionDefinition::RemoveKey(int nApplicationID, int nCommandType, int nKey)
{
	ASSERT(0 <= nApplicationID && nApplicationID < MAX_APP);

#ifdef MFCDLL
	for (POSITION pos = m_oKey[nApplicationID].GetHeadPosition(); pos; ) {
		POSITION currentPos = pos;
		KeyBind *pKeyBind = (KeyBind *)m_oKey[nApplicationID].GetNext(pos);
		if (pKeyBind->nCommandType == nCommandType && pKeyBind->bVk == nKey) {
			delete pKeyBind;
//			pKeyBind = NULL;
			m_oKey[nApplicationID].RemoveAt(currentPos);
			break;
		}
	}
#else
	vector<KeyBind>& k = m_oKey[nApplicationID];
	vector<KeyBind>::iterator i = k.begin();
	while (i != k.end()) {
		if (i->nCommandType == nCommandType && i->bVk == nKey) {
			k.erase(i);
			break;	// continue;	// ŏɈvGĝ
		}
		i++;
	}
#endif
}
